import os
from importlib import metadata
from pathlib import Path
from typing import Optional, Any

from ipfabric import IPFClient
from pydantic import BaseModel

IPF, CONFIGS = None, []


class ConfigFile(BaseModel):
    file: Path
    parsed: Optional[Any] = None
    raw: Optional[Any] = None


def pytest_addoption(parser):
    parser.addoption(
        "--config-dir",
        help="Path to directory with configurations.",
        action="store",
        type=Path,
    )
    parser.addoption(
        "--snapshot",
        help="Optional: IP Fabric Snapshot ID; defaults to `$last`.",
        action="store",
        default=None,
        type=str,
    )
    parser.addoption(
        "--ipf-url",
        help="IP Fabric URL: `https://demo1.us.ipfabric.io/`, (or use `IPF_URL` in`.env` file).",
        action="store",
        default=None,
        type=str,
    )
    parser.addoption(
        "--ipf-token",
        help="IP Fabric Token, (or use `IPF_TOKEN` in `.env` file).",
        action="store",
        default=None,
        type=str,
    )
    parser.addoption(
        "--insecure",
        help="Verify IP Fabric Certificate, defaults to `True`, (or use `IPF_VERIFY` in`.env` file).",
        dest='insecure',
        action="store_false",
    )


def pytest_configure(config):
    global IPF, CONFIGS

    if config.getoption("--config-dir"):
        cfg_dir = config.getoption("--config-dir").resolve()
        CONFIGS = [ConfigFile(file=cfg_dir / f) for f in os.listdir(cfg_dir)]
    else:
        IPF = IPFClient(
            base_url=config.getoption("--ipf-url"),
            auth=config.getoption("--ipf-token"),
            verify=config.getoption("--insecure", None),
            snapshot_id=config.getoption("--snapshot"),
        )
        IPF._client.headers['user-agent'] += f'; python-pynetcheck/{metadata.version("pynetcheck")}'



from pynetcheck.models.device import IPFDevice
from pynetcheck.tests.conftest import IPF


if not IPF:
    CISCO, ASA, IOS, IOSXE, IOSXR, NXOS = [], [], [], [], [], []
else:
    # Get all Cisco devices
    CISCO = [
        IPFDevice(ipf=IPF, device=d) for d in IPF.devices.by_vendor.get("cisco", [])
    ]
    ASA = [IPFDevice(ipf=IPF, device=d) for d in IPF.devices.by_family.get("asa", [])]
    IOS = [IPFDevice(ipf=IPF, device=d) for d in IPF.devices.by_family.get("ios", [])]
    IOSXE = [
        IPFDevice(ipf=IPF, device=d) for d in IPF.devices.by_family.get("ios-xe", [])
    ]
    IOSXR = [
        IPFDevice(ipf=IPF, device=d) for d in IPF.devices.by_family.get("ios-xr", [])
    ]
    NXOS = [
        IPFDevice(ipf=IPF, device=d) for d in IPF.devices.by_family.get("nx-os", [])
    ]



"""
https://security.paloaltonetworks.com/CVE-2024-3400
"""

import re

import pytest
from netutils.os_version import compare_version_loose

from pynetcheck.models.device import IPFDevice
from pynetcheck.tests.conftest import CONFIGS, ConfigFile
from pynetcheck.tests.paloalto.conftest import PA_FW

pytestmark = [pytest.mark.cve, pytest.mark.paloalto]

TELEMETRY = re.compile(r"\s?telemetry enable;")
GLOBAL_PROTECT = re.compile(
    r"^\s*global-protect\s*\{[^}]*(portals|gateways) enable;[^}]*",
    flags=re.DOTALL | re.M,
)


@pytest.mark.parametrize(
    "device", PA_FW, ids=[d.inventory.hostname for d in PA_FW], scope="class"
)
class TestPaloAltoTelemetryGlobalProtect:
    __test__ = True if PA_FW else False  # If no devices then skip

    @pytest.fixture(autouse=True, scope="class")
    def load_device_data(self, device: IPFDevice):
        if not device.loaded:
            device.load_data()
        if not device.config.current:
            pytest.skip("No configs for device.")
        yield
        device.clear_data()

    def test_device_telemetry_disabled(self, device: IPFDevice):
        if (
            (
                compare_version_loose(device.inventory.version, ">=", "11.1")
                and compare_version_loose(device.inventory.version, "<", "11.1.2-h3")
            )
            or (
                compare_version_loose(device.inventory.version, ">=", "11.0")
                and compare_version_loose(device.inventory.version, "<", "11.0.4-h1")
            )
            or (
                compare_version_loose(device.inventory.version, ">=", "10.2")
                and compare_version_loose(device.inventory.version, "<", "10.2.7-h8")
            )
            or (
                compare_version_loose(device.inventory.version, ">=", "10.2.8")
                and compare_version_loose(device.inventory.version, "<", "10.2.8-h3")
            )
            or (
                compare_version_loose(device.inventory.version, ">=", "10.2.9")
                and compare_version_loose(device.inventory.version, "<", "10.2.9-h1")
            )
        ):
            telemetry = bool(TELEMETRY.search(device.config.current))
            global_protect = bool(GLOBAL_PROTECT.search(device.config.current))
            assert not (telemetry and global_protect), (
                "Possibly Vulnerable: Device Telemetry enabled with GlobalProtect Gateway and/or Portal. "
                "Please upgrade device or disable telemetry."
            )
        pytest.skip(
            f"{device.inventory.hostname}:{device.inventory.sn} is not an affected SW Version {device.inventory.version}."
        )


@pytest.mark.parametrize(
    "config_file", CONFIGS, ids=[c.file.name for c in CONFIGS], scope="class"
)
class TestHTTPServerConfig(object):
    __test__ = True if CONFIGS else False  # If no configs then skip

    @pytest.fixture(autouse=True, scope="class")
    def load_configuration(self, config_file: ConfigFile):
        with open(config_file.file, "r") as file:
            config_file.raw = file.readlines()
        yield
        config_file.raw = None

    def test_device_telemetry_disabled(self, config_file):
        telemetry = bool(TELEMETRY.search(config_file.raw))
        global_protect = bool(GLOBAL_PROTECT.search(config_file.raw))
        assert not (telemetry and global_protect), (
            "Possibly Vulnerable: Device Telemetry enabled with GlobalProtect Gateway and/or Portal. "
            "Please upgrade device or disable telemetry."
        )



from typing import Tuple

from pynetcheck.models.device import IPFDevice


def return_http_config(device: IPFDevice, server: str) -> Tuple[str, str]:
    line = f"^ip http {server}"
    current = "enabled" if device.parsed_config.current.find_lines(line) else "disabled"
    startup = None
    if device.config.status != "saved" and device.parsed_config.startup:
        startup = (
            "enabled" if device.parsed_config.startup.find_lines(line) else "disabled"
        )
    return current, startup


def return_service_config(device: IPFDevice, service: str) -> Tuple[str, str]:
    line = f"^{service}"
    current = "enabled" if device.parsed_config.current.find_lines(line) else "disabled"
    startup = None
    if device.config.status != "saved" and device.parsed_config.startup:
        startup = (
            "enabled" if device.parsed_config.startup.find_lines(line) else "disabled"
        )
    return current, startup



from pynetcheck.models.device import IPFDevice
from pynetcheck.tests.conftest import IPF


if not IPF:
    PALOALTO, PRISMA, PA_FW, PANORAMA = [], [], [], []
else:
    # Get all Palo Alto devices
    PALOALTO = [
        IPFDevice(ipf=IPF, device=d) for d in IPF.devices.by_vendor.get("paloalto", [])
    ]
    PRISMA = [
        IPFDevice(ipf=IPF, device=d) for d in IPF.devices.by_family.get("ion", [])
    ]
    PA_FW = [
        IPFDevice(ipf=IPF, device=d)
        for d in sum(
            list(
                IPF.devices.by_vendor.search("paloalto")
                .sub_regex("model", r"^pa(?!norama).*")
                .values()
            ),
            [],
        )
    ]
    PANORAMA = [
        IPFDevice(ipf=IPF, device=d) for d in IPF.devices.by_model.get("panorama", [])
    ]



"""
https://sec.cloudapps.cisco.com/security/center/content/CiscoSecurityAdvisory/cisco-sa-iosxe-priv-esc-seAx6NLX
"""

import pytest
from ciscoconfparse import CiscoConfParse

from pynetcheck.models.device import IPFDevice
from pynetcheck.models.shared import return_service_config
from pynetcheck.tests.cisco.conftest import IOSXE
from pynetcheck.tests.conftest import CONFIGS, ConfigFile

pytestmark = [pytest.mark.cve, pytest.mark.cisco]


@pytest.mark.parametrize(
    "device", IOSXE, ids=[d.inventory.hostname for d in IOSXE], scope="class"
)
class TestNetConfYang:
    __test__ = True if IOSXE else False  # If no devices then skip

    @pytest.fixture(autouse=True, scope="class")
    def load_device_data(self, device: IPFDevice):
        if not device.loaded:
            device.load_data()
        if not device.parsed_config:
            pytest.skip("No configs for device.  Please check if `Saved Config Consistency` Discovery Task is Enabled.")
        yield
        device.clear_data()

    def test_netconf_yang_disabled(self, device: IPFDevice):
        current, startup = return_service_config(device, "netconf-yang")
        assert current == "disabled", "Running Config - netconf-yang Enabled"
        if device.config.status != "saved" and startup:
            assert startup == "disabled", "Startup Config - netconf-yang Enabled"


@pytest.mark.parametrize(
    "config_file", CONFIGS, ids=[c.file.name for c in CONFIGS], scope="class"
)
class TestNetConfYangConfig(object):
    __test__ = True if CONFIGS else False  # If no configs then skip

    @pytest.fixture(autouse=True, scope="class")
    def load_configuration(self, config_file: ConfigFile):
        with open(config_file.file, "r") as file:
            config_file.parsed = CiscoConfParse(
                file.readlines(), syntax="ios", ignore_blank_lines=False
            )

            print("SETUP", config_file.file.name)
        yield
        config_file.parsed = None
        print("TEARDOWN", config_file.file.name)

    def test_netconf_yang_disabled(self, config_file):
        line = "^netconf-yang"
        assert not config_file.parsed.find_lines(line), "Config - netconf-yang Enabled"



"""
https://sec.cloudapps.cisco.com/security/center/content/CiscoSecurityAdvisory/cisco-sa-iosxe-webui-privesc-j22SaA4z

This will test for the following:
    - HTTP(S) Server Disabled
    - If Enabled then test HTTP(S) Server Vulnerable
"""

import pytest
from ciscoconfparse import CiscoConfParse
from netutils.os_version import compare_version_loose

from pynetcheck.models.device import IPFDevice
from pynetcheck.models.shared import return_http_config
from pynetcheck.tests.cisco.conftest import IOSXE
from pynetcheck.tests.conftest import CONFIGS, ConfigFile

pytestmark = [pytest.mark.cve, pytest.mark.cisco]

RUNNING = "Running Config - HTTP"
STARTUP = "Startup Config - HTTP"


# Test each device for HTTP(S) Server configuration in running and startup configs
@pytest.mark.parametrize(
    "device", IOSXE, ids=[d.inventory.hostname for d in IOSXE], scope="class"
)
class TestHTTPServerIPF:
    __test__ = True if IOSXE else False  # If no devices then skip

    @pytest.fixture(autouse=True, scope="class")
    def load_device_data(self, device: IPFDevice):
        if not device.loaded:
            device.load_data()
        if not device.parsed_config:
            pytest.skip("No configs for device.  Please check if `Saved Config Consistency` Discovery Task is Enabled.")
        yield
        device.clear_data()

    @staticmethod
    def return_module_config(device: IPFDevice, server):
        current, startup = return_http_config(device, server)
        if current != "enabled" and startup != "enabled":
            pytest.skip(f"HTTP {server} Disabled")
        line = (
            "^ip http active-session-modules none"
            if server == "server"
            else "^ip http secure-active-session-modules none"
        )
        mcurrent = (
            "secured" if device.parsed_config.current.find_lines(line) else "vulnerable"
        )
        mstartup = None
        if device.config.status != "saved" and startup:
            mstartup = (
                "secured"
                if device.parsed_config.startup.find_lines(line)
                else "vulnerable"
            )
        return mcurrent, mstartup

    @staticmethod
    def check_version(device: IPFDevice):
        if (
            (compare_version_loose(device.inventory.version, "<=", "17.3.8a"))
            or (
                compare_version_loose(device.inventory.version, ">=", "17.4")
                and compare_version_loose(device.inventory.version, "<", "17.6.6a")
            )
            or (
                compare_version_loose(device.inventory.version, ">=", "17.7")
                and compare_version_loose(device.inventory.version, "<", "17.9.4a")
            )
        ):
            return True
        return False

    def test_http_server_disabled(self, device: IPFDevice):
        current, startup = return_http_config(device, "server")
        assert current == "disabled", f"{RUNNING} server Enabled"
        if device.config.status != "saved" and startup:
            assert startup == "disabled", f"{STARTUP} server Enabled"

    def test_https_server_disabled(self, device: IPFDevice):
        current, startup = return_http_config(device, "secure-server")
        assert current == "disabled", f"{RUNNING} secure-server Enabled"
        if device.config.status != "saved" and startup:
            assert startup == "disabled", f"{STARTUP} secure-server Enabled"

    def test_http_server_vulnerable(self, device: IPFDevice):
        if not self.check_version(device):
            pytest.skip(
                f"{device.inventory.hostname}:{device.inventory.sn} is not an affected SW Version {device.inventory.version}."
            )
        mcurrent, mstartup = self.return_module_config(device, "server")
        assert mcurrent == "secured", f"{RUNNING} server Vulnerable"
        if device.config.status != "saved" and mstartup:
            assert mstartup == "secured", f"{RUNNING} server Vulnerable"

    def test_https_server_vulnerable(self, device: IPFDevice):
        if not self.check_version(device):
            pytest.skip(
                f"{device.inventory.hostname}:{device.inventory.sn} is not an affected SW Version {device.inventory.version}."
            )
        mcurrent, mstartup = self.return_module_config(device, "secure-server")
        assert mcurrent == "secured", f"{RUNNING} server Vulnerable"
        if device.config.status != "saved" and mstartup:
            assert mstartup == "secured", f"{RUNNING} server Vulnerable"


# Test each config file in `dir` directory for HTTP(S) Server configuration
@pytest.mark.parametrize(
    "config_file", CONFIGS, ids=[c.file.name for c in CONFIGS], scope="class"
)
class TestHTTPServerConfig(object):
    __test__ = True if CONFIGS else False  # If no configs then skip

    @pytest.fixture(autouse=True, scope="class")
    def load_configuration(self, config_file: ConfigFile):
        with open(config_file.file, "r") as file:
            config_file.parsed = CiscoConfParse(
                file.readlines(), syntax="ios", ignore_blank_lines=False
            )
        yield
        config_file.parsed = None

    def test_http_server_disabled(self, config_file):
        assert config_file.parsed.find_lines(
            "^no ip http server"
        ), "HTTP server Enabled"

    def test_http_server_vulnerable(self, config_file):
        if config_file.parsed.find_lines("^no ip http server"):
            pytest.skip("HTTP server Disabled")
        assert config_file.parsed.find_lines(
            "^ip http active-session-modules none"
        ), "HTTP server Vulnerable"

    def test_https_server_disabled(self, config_file):
        assert config_file.parsed.find_lines(
            "^no ip http secure-server"
        ), "HTTP secure-server Enabled"

    def test_https_server_vulnerable(self, config_file):
        if config_file.parsed.find_lines("^no ip http secure-server"):
            pytest.skip("HTTP secure-server Disabled")
        assert config_file.parsed.find_lines(
            "^ip http secure-active-session-modules none"
        ), "HTTP secure-server Vulnerable"



import os
import sys

import pytest


def main():
    pytest.main(
        args=[
            os.path.join(os.path.dirname(os.path.abspath(__file__)), "tests"),
            *sys.argv[1:],
        ]
    )


if __name__ == "__main__":
    main()



import os

import pytest

from pynetcheck.models.device import IPFDevice
from pynetcheck.models.shared import return_http_config
from .conftest import CISCO


@pytest.mark.cisco
@pytest.mark.parametrize(
    "device", CISCO, ids=[d.inventory.hostname for d in CISCO], scope="class"
)
class TestCiscoServices:
    __test__ = True if CISCO else False

    @pytest.fixture(autouse=True, scope="class")
    def load_device_data(self, device: IPFDevice):
        if "ios" not in device.inventory.family:
            pytest.skip(
                f"Currently not implemented for family: {device.inventory.family}"
            )
        if not device.loaded:
            device.load_data()
        if not device.parsed_config:
            pytest.skip("No configs for device.")
        yield
        device.clear_data()

    @staticmethod
    def return_status(variable: str):
        check = (
            "enabled"
            if os.getenv(variable, "ENABLED").upper() == "DISABLED"
            else "disabled"
        )
        status = "Disabled" if check == "enabled" else "Enabled"
        return check, status

    def test_http_server(self, device: IPFDevice):
        """Set HTTP_SERVER=DISABLED to fail if HTTP(s) servers are disabled."""
        check, status = self.return_status("CISCO_HTTP_SERVER")
        current, startup = return_http_config(device, "server")
        assert current == check, f"Running Config - HTTP server {status}"
        if device.config.status != "saved" and startup:
            assert startup == check, f"Startup Config - HTTP server {status}"

    def test_https_server(self, device: IPFDevice):
        """Set HTTP_SERVER=DISABLED to fail if HTTP(s) servers are disabled."""
        check, status = self.return_status("CISCO_HTTPS_SERVER")
        current, startup = return_http_config(device, "secure-server")
        assert current == check, f"Running Config - HTTPS server {status}"
        if device.config.status != "saved" and startup:
            assert startup == check, f"Startup Config - HTTPS server {status}"

    def test_scp_server(self, device: IPFDevice):
        """Set SCP_SERVER=DISABLED to fail if SCP Server is disabled."""
        check, status = self.return_status("CISCO_SCP_SERVER")

        assert (
            "enabled"
            if device.parsed_config.current.find_lines("^ip scp server enable")
            else "disabled" == check
        ), f"Running Config - SCP Server {status}"
        if device.config.status != "saved" and device.parsed_config.startup:
            assert (
                "enabled"
                if device.parsed_config.startup.find_lines("^ip scp server enable")
                else "disabled" == check
            ), f"Startup Config - SCP Server {status}"



import re
from typing import Optional

from ciscoconfparse import CiscoConfParse
from ipfabric import IPFClient
from ipfabric.models.device import Device, DeviceConfig


def parse_family(family: str):
    if "ios" in family:
        return "ios"
    elif "asa" in family:
        return "asa"
    elif "nx-os" == family:
        return "nxos"
    elif "asa" == family:
        return "asa"
    else:
        return family


class ParsedConfig:
    def __init__(self, current: CiscoConfParse, startup: CiscoConfParse = None):
        self.current: CiscoConfParse = current
        self.startup: Optional[CiscoConfParse] = startup if startup else None


class IPFDevice:
    def __init__(self, ipf: IPFClient, device: Device):
        self.ipf: IPFClient = ipf
        self.inventory: Device = device
        self.site: str = device.site.upper()
        self.loaded: bool = False

        # Config will load when test is run and then be removed to saved memory.
        self.config: Optional[DeviceConfig] = None
        self.parsed_config: Optional[ParsedConfig] = None

    def __repr__(self):
        return f"{self.inventory.hostname}:{self.inventory.sn}"

    def load_data(self):
        self.get_config()
        self.loaded = True

    def clear_data(self):
        self.config = None
        self.parsed_config = None
        self.loaded = False

    def _cisco_config(self):
        current = (
            CiscoConfParse(
                self.config.current.split("\n"),
                syntax=parse_family(self.inventory.family),
                ignore_blank_lines=False,
            )
            if self.config.current
            else None
        )
        startup = (
            CiscoConfParse(
                self.config.start.split("\n"),
                syntax=parse_family(self.inventory.family),
                ignore_blank_lines=False,
            )
            if self.config.start
            else None
        )
        if self.config.current:
            self.parsed_config = ParsedConfig(current, startup)

    def _palo_alto_config(self):
        log = self.inventory.get_log_file()
        user = re.search(r"(\S*)@", log.split("\n")[0]).group(1)
        self.config = DeviceConfig(
            currentConfig=re.split(
                rf"^{user}", log.split("show config merged")[1], flags=re.M
            )[0].replace("\r\n", "\n"),
            startupConfig=None,
            status="N/A",
        )

    def get_config(self):
        if self.inventory.vendor == "cisco":
            self.config = self.inventory.get_config()
            self._cisco_config()
        elif self.inventory.vendor == "paloalto":
            self._palo_alto_config()
